# orbipay-paymentsapi-client

orbipay-paymentsapi-client ---> Node client for Orbipay Payments API.
The Payments API enables businesses to accept credit, debit, and bank account payments.

- API version: 1.0.0
- Package version: 1.0.0

## Requirements

- "logtown": "^2.2.0",
- "superagent": "3.5.2"

## Installation

#### npm

Download the library from ["here"](https://developers.orbipay.com/downloads/orbipay/payments/sdks/node/v1.0.0/orbipay-paymentsapi-client-1.0.0.tgz),
And please follow the below steps.
1. You need to have npm and Node installed in your system (Latest versions will be recommended).
2. Install this client package in you project.
        eg. - npm install <downloads_directory>/checkout-node-sdk-1.0.0.tgz

```shell
cd <path_to_project_directory>
npm install <downloads_directory>/checkout-node-sdk-1.0.0.tgz
```

## Getting Started

Please follow the [installation](#installation) instruction and use the JS code sample mentioned below as an example:

To see the logs Please [follow](https://github.com/logtown/logtown). For Example,

```javascript
const Logger = require('logtown');
Logger.addWrapper(function (id, level, stats, ...rest) {
    console.log(`${level} [${id}]`, ...rest);
});
```

This library provides a way to configure the http_timeout through the environment variable 'ORBIPAY_API_CLIENT_TIMEOUT_MILLIS'. For example in linux,

```shell
export ORBIPAY_API_CLIENT_TIMEOUT_MILLIS=60000
```

Please follow the [documentation](https://developers.orbipay.com/slate_doc_orbipay/#PaymentsAPI) to get the details on optional or mandatory properties.

```javascript
var Orbipay = require('orbipay-paymentsapi-client');

//Please use 'Orbipay.****Enum.**' for getting the valid values of a field. For example - Orbipay.PaymentAmountTypeEnum.current_balance for 'payment_amount_type'.

var channel = "channel_example"; // For Example: Web Portal
var clientKey = "clientKey_example"; // For Example: 34958763485
var idempotentRequestKey = "idempotentRequestKey_example"; // For Example: idempotent_2332487346875
var requestor = "requestor_example"; // For Example: 232343244
var secret = "secret"; // For Example: 9876784657655
var requestorType = "requestorType_Example";// For Example: Orbipay.RequestorTypeEnum.customer

var attributes = new Orbipay.Attributes(); // For Accepting X-OPAY headers. They will be accepted as key and values. 
attributes.add('example_key_1', 'example_value_1');
attributes.add('example_key_2', 'example_value_2');


var invocationContext = new Orbipay.InvocationContext(idempotentRequestKey, secret, attributes);

var payment_recurring_type = "payment_recurring_type_example"; // For Example: Orbipay.PaymentRecurringTypeEnum.monthly
var payment_recurring_count = "payment_recurring_count_example"; // For Example: 10
var payment_amount_type = "payment_recurring_count_example"; // For Example: Orbipay.PaymentAmountTypeEnum.current_balance
var payment_start_date = "payment_start_date_exmaple"; // For Example: 2018-08-13
var payment_end_date = "payment_end_date_exmaple"; // For Example: 2019-08-13
var payment_limit_amount = "payment_limit_amount_example"; // For Example: 100
var payment_plan_id = "payment_plan_id_example"; // For Example: 18273
var payment_defer_days = "payment_defer_days_example"; // For Example: 10

var payment_schedule = new Orbipay.PaymentSchedule(payment_recurring_type, payment_recurring_count,
    payment_amount_type, payment_start_date, payment_end_date,
    payment_limit_amount, payment_plan_id, payment_defer_days);

var amount = "amount_example"; //For Example: 200
var card_cvv_number = "card_cvv_number_example"; // For Example: 1234
var payment_setup_schedule_type = "payment_setup_schedule_type"; // For Example: Orbipay.PaymentSetupScheduleTypeEnum.autopay_enrollment
var fee_amount = "fee_amount_example"; //For Example: 5

var payment_setup_reference = "payment_setup_reference_example"; //For Example: payment_234345435
var comments = "comments_example"; //For Example: Creating PaymentSetup For Saikrishna.
var id_funding_account = "id_funding_account_example"; //For Example: 8347568734
var id_customer_account = "id_customer_account_example"; //For Example: 968798768
var id_customer = "id_customer_example"; //For Example: 232343244

var custom_fields = {'example_key1': 'example_value1'};

var callback = function (exception, data) {
    if (exception) {
        var http_response_code = exception['http_response_code'];
        var errors = exception['errors'];
        for (var i = 0; i < errors.length; i++) {
            var error = errors[i];
            console.log('code: '+ error['code']);
            console.log('message: '+ error['message']);
            console.log('field: '+ error['field']);
        }
    } else {
        console.log('API called successfully. Returned data: ' + JSON.stringify(data));
    }
};

new Orbipay.PaymentSetup()
    .withDetails(amount, card_cvv_number, payment_setup_schedule_type)
    .withCustomFields(custom_fields)
    .withFee(new Orbipay.Fee(fee_amount))
    .withPaymentSchedule(payment_schedule)
    .withReference(payment_setup_reference)
    .withMemo(comments)
    .fromAccount(new Orbipay.FundingAccount(id_funding_account))
    .toAccount(new Orbipay.CustomerAccount(id_customer_account))
    .forCustomer(id_customer)
    .createdBy(requestor, requestorType)
    .videChannel(channel)
    .forClient(clientKey)
    .create(invocationContext, callback); // for production
    //.create(invocationContext, callback, Orbipay.InvocationModeEnum.sandbox); // For sandbox
    //.create(invocationContext, callback, Orbipay.InvocationModeEnum.test, http://localhost:8080/payments/v1); // For local testing.
```

