/**
 * Created by Alacriti Inc on 8/3/18.
 */
(function (root, factory) {
    if (typeof module === 'object' && module.exports) {
        // CommonJS-like environments that support module.exports, like Node.
        module.exports = factory();
    }
}(this, function () {
    'use strict';


    /**
     * The HttpParamConstants model module.
     * @module constant/HttpParamConstants
     * @version 1.0.0
     */

    /**
     * Constructs a new <code>HttpParamConstants</code>.
     * @alias module:constants/HttpParamConstants
     * @class
     */
    var exports = Object.freeze({
        //Header Params
        CLIENT_KEY: 'client_key',
        AUTHORIZATION_HEADER: 'Authorization',
        PRODUCT: 'product',
        CHANNEL: 'channel',
        REQUESTOR: 'requestor',
        REQUESTOR_TYPE: 'requestor_type',
        TIMESTAMP: 'timestamp',
        IDEMPOTENT_REQUEST_KEY: 'idempotent_request_key',
        INVOCATION_CONTEXT: 'invocation_context',
        HEADER_CONTENT_TYPE: 'Content-Type',
        X_OPAY_HEADERS: 'X-OPAY-Headers',
        ACCEPT: 'accept',
        RESPONSE_CODES: 'response_codes',
        RESPONSE_TEXT: 'response_text',
        REQUEST_UUID: 'request_uuid',
        ATTRIBUTES: 'attributes',
        ATTRIBUTE: 'attribute',
        LIST: 'list',
        HEADER: 'header',
        SECRET: 'secret',
        ORBIPAY_REQUEST_HEADERS: ['client_key', 'product', 'channel', 'requestor',
            'requestor_type', 'timestamp', 'idempotent_request_key', 'X-OPAY-Headers'],
        SENSITIVE_KEYS: ['account_number', 'card_cvv_number', 'expiry_date', 'ssn'],
        REQUEST: 'request',
        DATA: 'data',
        EXCEPTION: 'exception',
        ERRORS: 'errors',

        // Added for new JSON REST Connect API
        CUSTOMER_REFERENCE: 'customer_reference',
        FIRST_NAME: 'first_name',
        LAST_NAME: 'last_name',
        MIDDLE_NAME: 'middle_name',
        GENDER: 'gender',
        SSN: 'ssn',
        LANGUAGE: 'language',
        EMAIL: 'email',
        ADDRESS_LINE1: 'address_line1',
        ADDRESS_LINE2: 'address_line2',
        ADDRESS_CITY: 'address_city',
        ADDRESS_STATE: 'address_state',
        ADDRESS_COUNTRY: 'address_country',
        ADDRESS_ZIP1: 'address_zip1',
        ADDRESS_ZIP2: 'address_zip2',
        ACCOUNT_TYPE: 'account_type',
        YES: 'yes',
        NO: 'no',
        DATE_OF_BIRTH: 'date_of_birth',
        HOME_PHONE: 'home_phone',
        WORK_PHONE: 'work_phone',
        MOBILE_PHONE: 'mobile_phone',
        CUSTOMER_ACCOUNTS: 'customer_accounts',
        FUNDING_ACCOUNTS: 'funding_accounts',
        PAYMENTS: 'payments',
        CUSTOMER: 'customer',
        ADDRESS: 'address',
        COMMENTS: 'comments',
        ID: 'id',
        STATUS: 'status',
        URL: 'url',

        //path params
        PATH_PARAM_ID_CUSTOMER: 'ID_CUSTOMER',
        PATH_PARAM_ID_FUNDING_ACCOUNT: 'ID_FUNDING_ACCOUNT',
        PATH_PARAM_ID_PAYMENT: 'ID_PAYMENT',
        PATH_PARAM_ID_CUSTOMER_ACCOUNT: 'ID_CUSTOMER_ACCOUNT',
        PATH_PARAM_ID_PAYMENT_SETUP: 'ID_PAYMENT_SETUP',
        PATH_PARAM_ID_FEE: 'ID_FEE',

        CARD_CVV_NUMBER: 'card_cvv_number',
        DELETED: 'deleted',
        PAYMENT_SETUP_SCHEDULE_TYPE: 'payment_setup_schedule_type',
        PAYMENT_RECURRING_TYPE: 'payment_recurring_type',
        PAYMENT_RECURRING_COUNT: 'payment_recurring_count',
        PAYMENT_AMOUNT_TYPE: 'payment_amount_type',
        PAYMENT_START_DATE: 'payment_start_date',
        PAYMENT_END_DATE: 'payment_end_date',
        PAYMENT_LIMIT_AMOUNT: 'payment_limit_amount',
        PAYMENT_SCHEDULE: 'payment_schedule',
        RETRIEVE_LIST: 'list',
        TOTAL_RESULTS_COUNT: 'total_results_count',
        HAS_MORE_RESULTS: 'has_more_results',
        FROM_INDEX: 'from_index',
        TO_INDEX: 'to_index',
        ACH_ELIGIBLE_FLAG: 'ach_eligible_flag',
        ACCOUNT_HOLDER_NAME: 'account_holder_name',
        NICKNAME: 'nickname',
        ACCOUNT_NUMBER: 'account_number',
        ISSUER_NAME: 'issuer_name',
        ABA_ROUTING_NUMBER: 'aba_routing_number',
        ACCOUNT_SUBTYPE: 'account_subtype',
        CURRENCY_CODE_3D: 'currency_code3d',
        EXPIRY_DATE: 'expiry_date',
        ACCOUNT_HOLDER_TYPE: 'account_holder_type',
        ACCOUNT_HOLDER_TYPE_PERSONAL: 'PERSONAL',
        CUSTOM_FIELDS: 'custom_fields',
        PAYMENT_REFERENCE: 'payment_reference',
        AMOUNT: 'amount',
        REFUND_AMOUNT: 'refund_amount',
        PAYMENT_DATE: 'payment_date',
        FEE: 'fee',
        FEE_AMOUNT: 'fee_amount',
        FEE_TYPE: 'fee_type',
        PAYER: 'payer',
        PAYEE: 'payee',
        FUNDING_ACCOUNT: 'funding_account',
        CUSTOMER_ACCOUNT: 'customer_account',
        CONFIRMATION_NUMBER: 'confirmation_number',
        PAYMENT_ENTRY_DATE: 'payment_entry_date',
        PAYMENT_RETURN_DATE: 'payment_return_date',
        RETURN_CODE: 'return_code',
        CURRENT_BALANCE: 'current_balance',
        CURRENT_STATEMENT_BALANCE: 'current_statement_balance',
        MINIMUM_PAYMENT_DUE: 'minimum_payment_due',
        PAST_AMOUNT_DUE: 'past_amount_due',
        PAYMENT_DUE_DATE: 'payment_due_date',
        ATM_ELIGIBLE_FLAG: 'atm_eligible_flag',
        PAYMENT_METHOD: 'payment_method',
        PAYMENT_SCHEDULE_TYPE: 'payment_schedule_type',
        PAYMENT_PLAN_ID: 'payment_plan_id',
        PAYMENT_DEFER_DAYS: 'payment_defer_days',

        RECURRING_PAYMENT_AMOUNT_TYPE: 'recurring_payment_amount_type',
        STATEMENT_DATE: 'statement_date',
        PAYMENT_REQUEST_DATE: 'payment_request_date',

        PAGE_SIZE: 'page_size',
        QUERY_ID: 'query_id',
        FROM_DATE: 'from_date',
        TO_DATE: 'to_date',
        VARIABLE_RECURRING_ENROLMENT: 'variable_recurring_enrolment',
        AUTOPAY_ENROLLMENT: 'autopay_enrollment',

        // Customer Accounts info
        CUSTOMER_ACCOUNT_REFERENCE: 'customer_account_reference',
        // Funding sources info
        CURRENCY_CODE3D: 'currency_code3d',
        CARD_EXPIRY_DATE: 'expiry_date',
        LOCALE: 'locale',
        // Payment info
        PAYMENT: 'payment',
        PAYMENT_SETUP: 'payment_setup',
        PAYER_ID: 'Payer[id]',
        TRANSACTION_TYPE: 'transaction_type',
        PAYMENT_SETUP_REFERENCE: 'payment_setup_reference',
        _TXN_PYMNT_SCHLD_TYPE: '_txn_pymnt_schld_type',

        // Refund info
        REFUND: 'refund',
        REFUND_REFERENCE: 'refund_reference',
        REFUND_TYPE: 'refund_type',


        // Query Paramertes names
        QUERY_PARAM_ID_CUSTOMER: 'id_customer',
        QUERY_PARAM_ID_FUNDING_ACCOUNT: 'id_funding_account',
        QUERY_PARAM_ID_CUSTOMER_ACCOUNT: 'id_customer_account',

        TRUE: 'true',
        FALSE: 'false',

        API_END_POINT: 'api_end_point',

        //symbols
        DOT: '.',
        LEFT_SQUARE_BRACE: '[',
        RIGHT_SQUARE_BRACE: ']'
    });

    return exports;
}));
