/**
 * Created by Alacriti Inc on 8/3/18.
 */
(function (root, factory) {
    if (typeof module === 'object' && module.exports) {
        // CommonJS-like environments that support module.exports, like Node.
        module.exports = factory(require('logtown').getLogger('orbipay-paymentsapi-client/handler/customer/GetCustomerHandler'), require('../../swagger/ApiClient'), require('./../../constant/RestConstants'), require('../../swagger/api/CustomerApi'), require('./../../util/CommonUtil'), require('../../exception/OrbipayApiException'), require('./../../util/ValidationUtil'), require('./../../constant/HttpParamConstants'), require('./../../constant/AppErrorCodes'), require('./../../enum/DataTypeEnum'), require('./../../enum/MandatoryEnum'));
    }
}(this, function (logger, ApiClient, RestConstants, CustomerApi, CommonUtil, OrbipayApiException, ValidationUtil, HttpParamConstants, AppErrorCodes, DataTypeEnum, MandatoryEnum) {
    'use strict';


    /**
     * The GetCustomerHandler model module.
     * @module handler/customer/GetCustomerHandler
     * @version 1.0.0
     */

    /**
     * Constructs a new <code>GetCustomerHandler</code>.
     * @alias module:handler/customer/GetCustomerHandler
     * @class
     */
    var exports = function () {
    };


    exports.prototype.handleRequest = function (customer, callback) {
        var response = {};
        var exception;
        try {
            exception = this.validate(customer);
            if (exception[HttpParamConstants.ERRORS].length > 0) {
                if (callback) {
                    callback(null, null, null, exception);
                }
                response[HttpParamConstants.EXCEPTION] = exception;
                return response;
            } else {
                return this.callAPI(customer, callback);
            }
        } catch (e) {
            exception = OrbipayApiException.getDefaultException(e, customer);
            if (callback) {
                callback(null, null, null, exception);
            }
            response[HttpParamConstants.EXCEPTION] = exception;
            return response;
        }
    };

    exports.prototype.validate = function (customer) {
        var exception = new OrbipayApiException(RestConstants.CLIENT_SIDE_FIELD_VALIDATION_FAILURE_MESSAGE, customer);
        ValidationUtil.validate(customer[HttpParamConstants.PATH_PARAM_ID_CUSTOMER], DataTypeEnum.String, MandatoryEnum.yes, exception,
            HttpParamConstants.PATH_PARAM_ID_CUSTOMER, AppErrorCodes.ID_CUSTOMER_INVALID, ValidationUtil.isValidId);

        ValidationUtil.validateInvocationContext(customer[HttpParamConstants.INVOCATION_CONTEXT], exception);

        ValidationUtil.validate(customer[HttpParamConstants.API_END_POINT], DataTypeEnum.String, MandatoryEnum.yes, exception,
            HttpParamConstants.API_END_POINT, AppErrorCodes.API_END_POINT_INVALID, ValidationUtil.isValidApiEndPoint);


        return exception;
    };

    exports.prototype.getOptions = function (customer) {
        return {
            'xOPAYHeaders': CommonUtil.getXOPAYHeaders(customer[HttpParamConstants.INVOCATION_CONTEXT][HttpParamConstants.ATTRIBUTES])
        };
    };

    exports.prototype.callAPI = function (customer, callback) {
        var invocationContext = customer[HttpParamConstants.INVOCATION_CONTEXT];
        var channel = invocationContext[HttpParamConstants.CHANNEL];
        var clientKey = invocationContext[HttpParamConstants.CLIENT_KEY];
        var product = invocationContext[HttpParamConstants.PRODUCT];
        var timestamp = CommonUtil.getTimeStamp();
        var idempotentRequestKey = invocationContext[HttpParamConstants.IDEMPOTENT_REQUEST_KEY];
        var requestor = invocationContext[HttpParamConstants.REQUESTOR];
        var requestorType = invocationContext[HttpParamConstants.REQUESTOR_TYPE];

        var opts = this.getOptions(customer);

        var apiClient = new ApiClient(clientKey, invocationContext[HttpParamConstants.SECRET], customer[HttpParamConstants.API_END_POINT]);
        var customerApi = new CustomerApi(apiClient);
        var request = customerApi.getCustomer(channel, clientKey, product, timestamp, idempotentRequestKey, requestor, requestorType,
            customer[HttpParamConstants.PATH_PARAM_ID_CUSTOMER], opts, callback);

        var response = {};
        response[HttpParamConstants.REQUEST] = request;
        return response;
    };
    return exports;
}));


