/**
 * Created by Alacriti Inc on 8/3/18.
 */
(function (root, factory) {
    if (typeof module === 'object' && module.exports) {
        // CommonJS-like environments that support module.exports, like Node.
        module.exports = factory(require('logtown').getLogger('orbipay-paymentsapi-client/handler/fundingaccount/DeleteFundingAccountHandler'), require('../../swagger/ApiClient'), require('./../../constant/RestConstants'), require('../../swagger/api/FundingAccountApi'), require('./../../util/CommonUtil'), require('../../exception/OrbipayApiException'), require('./../../util/ValidationUtil'), require('./../../constant/HttpParamConstants'), require('./../../constant/AppErrorCodes'), require('./../../enum/DataTypeEnum'), require('./../../enum/MandatoryEnum'), require('../../swagger/model/DeleteFundingAccountRequest'));
    }
}(this, function (logger, ApiClient, RestConstants, FundingAccountApi, CommonUtil, OrbipayApiException, ValidationUtil, HttpParamConstants, AppErrorCodes, DataTypeEnum, MandatoryEnum, DeleteFundingAccountRequest) {
    'use strict';


    /**
     * The DeleteFundingAccountHandler model module.
     * @module handler/fundingaccount/DeleteFundingAccountHandler
     * @version 1.0.0
     */

    /**
     * Constructs a new <code>DeleteFundingAccountHandler</code>.
     * @alias module:handler/fundingaccount/DeleteFundingAccountHandler
     * @class
     */
    var exports = function () {
    };


    exports.prototype.handleRequest = function (fundingAccount, callback) {
        var response = {};
        var exception;
        try {
            exception = this.validate(fundingAccount);
            if (exception[HttpParamConstants.ERRORS].length > 0) {
                if (callback) {
                    callback(null, null, null, exception);
                }
                response[HttpParamConstants.EXCEPTION] = exception;
                return response;
            } else {
                return this.callAPI(fundingAccount, callback);
            }
        } catch (e) {
            exception = OrbipayApiException.getDefaultException(e, fundingAccount);
            if (callback) {
                callback(null, null, null, exception);
            }
            response[HttpParamConstants.EXCEPTION] = exception;
            return response;
        }
    };

    exports.prototype.validate = function (fundingAccount) {
        var exception = new OrbipayApiException(RestConstants.CLIENT_SIDE_FIELD_VALIDATION_FAILURE_MESSAGE, fundingAccount);

        ValidationUtil.validate(fundingAccount[HttpParamConstants.PATH_PARAM_ID_CUSTOMER], DataTypeEnum.String, MandatoryEnum.yes, exception,
            HttpParamConstants.PATH_PARAM_ID_CUSTOMER, AppErrorCodes.ID_CUSTOMER_INVALID, ValidationUtil.isValidId);

        ValidationUtil.validate(fundingAccount[HttpParamConstants.PATH_PARAM_ID_FUNDING_ACCOUNT], DataTypeEnum.String, MandatoryEnum.yes, exception,
            HttpParamConstants.PATH_PARAM_ID_FUNDING_ACCOUNT, AppErrorCodes.ID_FUNDING_ACCOUNT_INVALID, ValidationUtil.isValidId);

        ValidationUtil.validateInvocationContext(fundingAccount['invocation_context'], exception);

        ValidationUtil.validate(fundingAccount[HttpParamConstants.API_END_POINT], DataTypeEnum.String, MandatoryEnum.yes, exception,
            HttpParamConstants.API_END_POINT, AppErrorCodes.API_END_POINT_INVALID, ValidationUtil.isValidApiEndPoint);

        return exception;
    };

    exports.prototype.getOptions = function (fundingAccount) {
        return {
            'xOPAYHeaders': CommonUtil.getXOPAYHeaders(fundingAccount[HttpParamConstants.INVOCATION_CONTEXT][HttpParamConstants.ATTRIBUTES])
        };
    };

    exports.prototype.callAPI = function (fundingAccount, callback) {
        var invocationContext = fundingAccount[HttpParamConstants.INVOCATION_CONTEXT];
        var channel = invocationContext[HttpParamConstants.CHANNEL];
        var clientKey = invocationContext[HttpParamConstants.CLIENT_KEY];
        var product = invocationContext[HttpParamConstants.PRODUCT];
        var timestamp = CommonUtil.getTimeStamp();
        var idempotentRequestKey = invocationContext[HttpParamConstants.IDEMPOTENT_REQUEST_KEY];
        var requestor = invocationContext[HttpParamConstants.REQUESTOR];
        var requestorType = invocationContext[HttpParamConstants.REQUESTOR_TYPE];

        var deleteFundingAccountRequest = DeleteFundingAccountRequest.constructFromObject(fundingAccount);

        var opts = this.getOptions(fundingAccount);

        var apiClient = new ApiClient(clientKey, invocationContext[HttpParamConstants.SECRET], fundingAccount[HttpParamConstants.API_END_POINT]);
        var fundingAccountApi = new FundingAccountApi(apiClient);
        var request = fundingAccountApi.deleteFundingAccount(channel, clientKey, product, timestamp, idempotentRequestKey, requestor, requestorType,
            fundingAccount[HttpParamConstants.PATH_PARAM_ID_FUNDING_ACCOUNT], fundingAccount[HttpParamConstants.PATH_PARAM_ID_CUSTOMER], deleteFundingAccountRequest, opts, callback);

        var response = {};
        response[HttpParamConstants.REQUEST] = request;
        return response;
    };
    return exports;
}));


