/**
 * Created by Alacriti Inc on 8/3/18.
 */
(function (root, factory) {
    if (typeof module === 'object' && module.exports) {
        // CommonJS-like environments that support module.exports, like Node.
        module.exports = factory(require('logtown').getLogger('orbipay-paymentsapi-client/model/base/Attribute'));
    }
}(this, function (logger) {
    'use strict';


    /**
     * The Attribute model module.
     * @module model/base/Attribute
     * @version 1.0.0
     */

    /**
     * Constructs a new <code>Attribute</code>.
     * @alias module:model/base/Attribute
     * @class
     */
    var exports = function (key, value) {
        var _this = this;

        _this['key'] = key;
        _this['value'] = value;
    };

    /**
     * @private
     * Constructs a <code>Attribute</code> from a plain JavaScript object, optionally creating a new instance.
     * Copies all relevant properties from <code>data</code> to <code>obj</code> if supplied or a new instance if not.
     * @param {Object} data The plain JavaScript object bearing properties of interest.
     * @param {module:model/base/Attribute} obj Optional instance to populate.
     * @return {module:model/base/Attribute} The populated <code>Attribute</code> instance.
     */
    exports.constructFromObject = function (data, obj) {
        if (data) {
            obj = obj || new exports();
            if (data.hasOwnProperty('key')) {
                obj['key'] = data['key'];
            }
            if (data.hasOwnProperty('value')) {
                obj['value'] = data['value'];
            }
        }
        return obj;
    };

    /**
     * @member {String} key
     */
    exports.prototype['key'] = undefined;
    /**
     * @member {String} value
     */
    exports.prototype['value'] = undefined;


    return exports;
}));


