/**
 * Created by Alacriti Inc on 8/3/18.
 */
(function (root, factory) {
    if (typeof module === 'object' && module.exports) {
        // CommonJS-like environments that support module.exports, like Node.
        module.exports = factory(require('logtown').getLogger('orbipay-paymentsapi-client/model/base/Attributes'), require('../../swagger/ApiClient'), require('./Attribute'));
    }
}(this, function (logger, ApiClient, Attribute) {
    'use strict';


    /**
     * The Attributes model module.
     * @module model/base/Attributes
     * @version 1.0.0
     */

    /**
     * Constructs a new <code>Attributes</code>.
     * @alias module:model/base/Attributes
     * @class
     */
    var exports = function () {
        var _this = this;
        _this['list'] = [];
    };

    /**
     * @private
     * Constructs a <code>Attributes</code> from a plain JavaScript object, optionally creating a new instance.
     * Copies all relevant properties from <code>data</code> to <code>obj</code> if supplied or a new instance if not.
     * @param {Object} data The plain JavaScript object bearing properties of interest.
     * @param {module:model/base/Attributes} obj Optional instance to populate.
     * @return {module:model/base/Attributes} The populated <code>Attributes</code> instance.
     */
    exports.constructFromObject = function (data, obj) {
        if (data) {
            obj = obj || new exports();
            if (data.hasOwnProperty('list')) {
                obj['list'] = ApiClient.convertToType(data['list'], [Attribute]);
            }
        }
        return obj;
    };

    /**
     * Add key and value to the attributes.
     * @param {String} key
     * @param {String} value
     */
    exports.prototype.add = function (key, value) {
        var _this = this;
        _this['list'].push(new Attribute(key, value));
    };

    /**
     * @private
     * Constructs a <code>Attributes</code> from xOPAYHeaders.
     * @param {String} xOPAYHeaders.
     * @param {module:model/base/Attributes} obj Optional instance to populate.
     * @return {module:model/base/Attributes} The populated <code>Attributes</code> instance.
     */
    exports.constructFromXOPAYHeaders = function (xOPAYHeaders, obj) {
        if (typeof xOPAYHeaders !== 'undefined' && xOPAYHeaders !== null && xOPAYHeaders !== '') {
            obj = obj || new exports();
            var xOPAYHeadersArray = xOPAYHeaders.split(';');
            var list = [];
            for (var i = 0; i < xOPAYHeadersArray.length; i++) {
                var xOPAYHeader = xOPAYHeadersArray[i];
                if (typeof xOPAYHeader !== 'undefined' && xOPAYHeader !== null && xOPAYHeader !== '') {
                    var index = xOPAYHeader.indexOf('=');
                    if (index > 0) {
                        var key = xOPAYHeader.substring(0, index);
                        if (index + 1 < xOPAYHeader.length) {
                            var value = xOPAYHeader.substring(index + 1, xOPAYHeader.length);
                            list.push(new Attribute(key, value));
                        }
                    }
                }
            }
            obj['list'] = list;
        }
        return obj;
    };


    /**
     * @member {Array.<module:model/base/Attribute>} list
     */
    exports.prototype['list'] = undefined;

    return exports;
}));