/**
 * Created by Alacriti Inc on 8/3/18.
 */
(function (root, factory) {
    if (typeof module === 'object' && module.exports) {
        // CommonJS-like environments that support module.exports, like Node.
        module.exports = factory(require('logtown').getLogger('orbipay-paymentsapi-client/model/payment/Payment'), require('./../customeraccount/CustomerAccount'), require('./../customer/Customer'), require('./Fee'), require('./../fundingaccount/FundingAccount'), require('./../base/Base'), require('./../base/InvocationContext'), require('./../../enum/InvocationModeEnum'), require('./../../constant/RestConstants'), require('./../../enum/ProductEnum'), require('./../base/Attributes'), require('./../../handler/payment/CreatePaymentHandler'), require('./../../handler/payment/GetPaymentHandler'), require('../../exception/OrbipayApiException'), require('./../../util/ValidationUtil'), require('./../../util/CommonUtil'), require('./../../handler/payment/UpdatePaymentHandler'), require('./../../handler/payment/DeletePaymentHandler'), require('./../../util/ResponseWrapper'), require('./PaymentNetworkResponse'));
    }
}(this, function (logger, CustomerAccount, Customer, Fee, FundingAccount, Base, InvocationContext, InvocationModeEnum, RestConstants, ProductEnum, Attributes, CreatePaymentHandler, GetPaymentHandler, OrbipayApiException, ValidationUtil, CommonUtil, UpdatePaymentHandler, DeletePaymentHandler, ResponseWrapper, PaymentNetworkResponse) {
    'use strict';


    /**
     * The Payment model module.
     * @module model/payment/Payment
     * @version 1.0.0
     */

    /**
     * Constructs a new <code>Payment</code>.
     * @alias module:model/payment/Payment
     * @class
     * @param ID_PAYMENT {String} The unique identifier assigned by Orbipay to the payment.
     */
    var exports = function (ID_PAYMENT) {
        var _this = this;
        Base.call(_this);
        _this['ID_PAYMENT'] = ID_PAYMENT;
        _this['id'] = ID_PAYMENT;
    };

    /**
     * @private
     * Constructs a <code>Payment</code> from a plain JavaScript object, optionally creating a new instance.
     * Copies all relevant properties from <code>data</code> to <code>obj</code> if supplied or a new instance if not.
     * @param {Object} data The plain JavaScript object bearing properties of interest.
     * @param {module:model/payment/Payment} obj Optional instance to populate.
     * @return {module:model/payment/Payment} The populated <code>Payment</code> instance.
     */
    exports.constructFromObject = function (data, obj) {
        if (data) {
            obj = obj || new exports();
            if (data.hasOwnProperty('id')) {
                obj['id'] = data['id'];
            }
            if (data.hasOwnProperty('url')) {
                obj['url'] = data['url'];
            }
            if (data.hasOwnProperty('fee')) {
                obj['fee'] = Fee.constructFromObject(data['fee']);
            }
            if (data.hasOwnProperty('status')) {
                obj['status'] = data['status'];
            }
            if (data.hasOwnProperty('payment_method')) {
                obj['payment_method'] = data['payment_method'];
            }
            if (data.hasOwnProperty('confirmation_number')) {
                obj['confirmation_number'] = data['confirmation_number'];
            }
            if (data.hasOwnProperty('currency_code3d')) {
                obj['currency_code3d'] = data['currency_code3d'];
            }
            if (data.hasOwnProperty('amount')) {
                obj['amount'] = data['amount'];
            }
            if (data.hasOwnProperty('card_cvv_number')) {
                obj['card_cvv_number'] = data['card_cvv_number'];
            }
            if (data.hasOwnProperty('payment_date')) {
                obj['payment_date'] = data['payment_date'];
            }
            if (data.hasOwnProperty('payment_schedule_type')) {
                obj['payment_schedule_type'] = data['payment_schedule_type'];
            }
            if (data.hasOwnProperty('payment_request_date')) {
                obj['payment_request_date'] = data['payment_request_date'];
            }
            if (data.hasOwnProperty('payment_amount_type')) {
                obj['payment_amount_type'] = data['payment_amount_type'];
            }
            if (data.hasOwnProperty('payment_reference')) {
                obj['payment_reference'] = data['payment_reference'];
            }
            if (data.hasOwnProperty('expected_payment_settlement_date')) {
                obj['expected_payment_settlement_date'] = data['expected_payment_settlement_date'];
            }
            if (data.hasOwnProperty('payment_entry_date')) {
                obj['payment_entry_date'] = data['payment_entry_date'];
            }
            if (data.hasOwnProperty('payment_return_date')) {
                obj['payment_return_date'] = data['payment_return_date'];
            }
            if (data.hasOwnProperty('return_code')) {
                obj['return_code'] = data['return_code'];
            }
            if (data.hasOwnProperty('customer')) {
                obj['customer'] = Customer.constructFromObject(data['customer']);
            }
            if (data.hasOwnProperty('funding_account')) {
                obj['funding_account'] = FundingAccount.constructFromObject(data['funding_account']);
            }
            if (data.hasOwnProperty('customer_account')) {
                obj['customer_account'] = CustomerAccount.constructFromObject(data['customer_account']);
            }
            if (data.hasOwnProperty('custom_fields')) {
                obj['custom_fields'] = data['custom_fields'];
            }
            if (data.hasOwnProperty('comments')) {
                obj['comments'] = data['comments'];
            }
            if (data.hasOwnProperty('deleted')) {
                obj['deleted'] = data['deleted'];
            }
            if (data.hasOwnProperty('payment_network_response')) {
                obj['payment_network_response'] = PaymentNetworkResponse.constructFromObject(data['payment_network_response']);
            }
        }
        return obj;
    };

    exports.prototype = Object.create(Base.prototype);

    exports.prototype.constructor = exports;

    /**
     * The unique identifier assigned by Orbipay to the payment.
     * @member {String} id
     */
    exports.prototype['id'] = undefined;
    /**
     * This URL fetches the details of payment.
     * @member {String} url
     */
    exports.prototype['url'] = undefined;
    /**
     * @member {module:model/payment/Fee} fee
     */
    exports.prototype['fee'] = undefined;
    /**
     * The status of the payment.
     * @member {module:model/payment/Payment.StatusEnum} status
     */
    exports.prototype['status'] = undefined;
    /**
     * The fulfillment method for the payment.
     * @member {module:model/payment/Payment.PaymentMethodEnum} payment_method
     */
    exports.prototype['payment_method'] = undefined;
    /**
     * The confirmation number or reference provided to the customer for the payment.
     * @member {String} confirmation_number
     */
    exports.prototype['confirmation_number'] = undefined;
    /**
     * The three-letter currency code in the ISO4217 format, in uppercase, for the card account. It must be a currency supported by Orbipay. This defaults to USD.
     * @member {String} currency_code3d
     */
    exports.prototype['currency_code3d'] = undefined;
    /**
     * The amount being paid. In case the payment_amount_type is 'discounted_amount', 'penalty_amount' or 'other', the amount needs to be specified. In other cases, this is derived from the payment_amount_type.
     * @member {String} amount
     */
    exports.prototype['amount'] = undefined;
    /**
     * This is never returned by Orbipay. This needs to be passed to Orbipay when making or editing a payment with a card funding account.
     * @member {String} card_cvv_number
     */
    exports.prototype['card_cvv_number'] = undefined;
    /**
     * The date on which the payment is scheduled to be made. It must be in ISO8601 full-date format, namely, YYYY-MM-DD.
     * @member {String} payment_date
     */
    exports.prototype['payment_date'] = undefined;
    /**
     * The schedule type for the payment. This defaults to one_time_payment.
     * @member {module:model/payment/Payment.PaymentScheduleTypeEnum} payment_schedule_type
     */
    exports.prototype['payment_schedule_type'] = undefined;
    /**
     * The date on which the biller wants the payment to be settled. It must be in ISO8601 full-date format, namely, YYYY-MM-DD. This is applicable only if the client has opted for it.
     * @member {String} payment_request_date
     */
    exports.prototype['payment_request_date'] = undefined;
    /**
     * The type of amount, as per the bill/statement, that is being paid. The payment amount is derived from the amount type selected for the payment. This defaults to other.
     * @member {module:model/payment/Payment.PaymentAmountTypeEnum} payment_amount_type
     */
    exports.prototype['payment_amount_type'] = undefined;
    /**
     * The unique identifier in the client system for the payment.
     * @member {String} payment_reference
     */
    exports.prototype['payment_reference'] = undefined;
    /**
     * The date on which the payment is expected to be settled with the biller. It must be in ISO8601 full-date format, namely, YYYY-MM-DD.
     * @member {String} expected_payment_settlement_date
     */
    exports.prototype['expected_payment_settlement_date'] = undefined;
    /**
     * The date on which the payment is captured in Orbipay. It must be in ISO8601 full-date format, namely, YYYY-MM-DD.
     * @member {String} payment_entry_date
     */
    exports.prototype['payment_entry_date'] = undefined;
    /**
     * The date on which the ACH returns is received by Orbipay.
     * @member {String} payment_return_date
     */
    exports.prototype['payment_return_date'] = undefined;
    /**
     * The ACH return code or the chargeback code received from Visa/Mastercard, in the event of returned payments.
     * @member {String} return_code
     */
    exports.prototype['return_code'] = undefined;
    /**
     * @member {module:model/customer/Customer} customer
     */
    exports.prototype['customer'] = undefined;
    /**
     * @member {module:model/fundingaccount/FundingAccount} funding_account
     */
    exports.prototype['funding_account'] = undefined;
    /**
     * @member {module:model/customeraccount/CustomerAccount} customer_account
     */
    exports.prototype['customer_account'] = undefined;
    /**
     * The additional information or meta-information that Orbipay can accept, maintain and transmit back to the client. The custom fields need to be configured with Orbipay before they can be accepted. Orbipay would reject custom fields that are not pre-configured. Please contact support@billerpayments.com for more information on configuring and using custom fields.
     * @member {Object} custom_fields
     */
    exports.prototype['custom_fields'] = undefined;
    /**
     * @member {String} deleted
     */
    exports.prototype['deleted'] = undefined;
    /**
     * @member {module:model/payment/PaymentNetworkResponse} payment_network_response
     */
    exports.prototype['payment_network_response'] = undefined;

    /**
     * @param {String} payment_reference
     * @return module:model/payment/Payment
     */
    exports.prototype.withReference = function (payment_reference) {
        var _this = this;
        _this['payment_reference'] = payment_reference;
        return _this;
    };

    /**
     * @param {Object} custom_fields
     * @return module:model/payment/Payment
     */
    exports.prototype.withCustomFields = function (custom_fields) {
        var _this = this;
        _this['custom_fields'] = custom_fields;
        return _this;
    };

    /**
     * @param {module:model/payment/Fee} fee.
     * @return module:model/payment/Payment
     */
    exports.prototype.withFee = function (fee) {
        var _this = this;
        _this['fee'] = Fee.constructFromObject(fee);
        return _this;
    };

    /**
     * @param {module:model/fundingaccount/FundingAccount} fundingAccount.
     * @return module:model/payment/Payment
     */
    exports.prototype.fromAccount = function (fundingAccount) {
        var _this = this;
        _this['funding_account'] = FundingAccount.constructFromObject(fundingAccount);
        return _this;
    };

    /**
     * @param {module:model/customeraccount/CustomerAccount} customerAccount.
     * @return module:model/payment/Payment
     */
    exports.prototype.toAccount = function (customerAccount) {
        var _this = this;
        _this['customer_account'] = CustomerAccount.constructFromObject(customerAccount);
        return _this;
    };

    /**
     * @param {String} comments.
     * @return module:model/payment/Payment
     */
    exports.prototype.withMemo = function (comments) {
        var _this = this;
        _this['comments'] = comments;
        return _this;
    };

    /**
     * @param {String} amount
     * @param {String} payment_date
     * @param {String} card_cvv_number
     * @param {String} payment_amount_type
     * @param {String} payment_request_date
     * @return module:model/payment/Payment
     */
    exports.prototype.withDetails = function (amount, payment_date, card_cvv_number, payment_amount_type, payment_request_date) {
        var _this = this;
        _this['amount'] = amount;
        _this['payment_date'] = payment_date;
        _this['card_cvv_number'] = card_cvv_number;
        _this['payment_amount_type'] = payment_amount_type;
        _this['payment_request_date'] = payment_request_date;
        return _this;
    };

    /**
     * @param {String} ID_CUSTOMER
     * @return module:model/payment/Payment
     */
    exports.prototype.forCustomer = function (ID_CUSTOMER) {
        var _this = this;
        _this['ID_CUSTOMER'] = ID_CUSTOMER;
        _this['customer'] = new Customer(ID_CUSTOMER);
        return _this;
    };

    /**
     * @param {String} requestor
     * @param {String} requestor_type
     * @return module:model/payment/Payment
     */
    exports.prototype.deletedBy = function (requestor, requestor_type) {
        var _this = this;
        _this.requestedBy(requestor, requestor_type);
        return _this;
    };

    /**
     * @param {String} requestor
     * @param {String} requestor_type
     * @return module:model/payment/Payment
     */
    exports.prototype.createdBy = function (requestor, requestor_type) {
        var _this = this;
        _this.requestedBy(requestor, requestor_type);
        return _this;
    };

    /**
     * @param {String} requestor
     * @param {String} requestor_type
     * @return module:model/payment/Payment
     */
    exports.prototype.retrievedBy = function (requestor, requestor_type) {
        var _this = this;
        _this.requestedBy(requestor, requestor_type);
        return _this;
    };

    /**
     * @param {String} requestor
     * @param {String} requestor_type
     * @return module:model/payment/Payment
     */
    exports.prototype.updatedBy = function (requestor, requestor_type) {
        var _this = this;
        _this.requestedBy(requestor, requestor_type);
        return _this;
    };

    /**
     * @param {String} channel
     * @return module:model/payment/Payment
     */
    exports.prototype.videChannel = function (channel) {
        var _this = this;
        Base.prototype.videChannel.call(_this, channel);
        return _this;
    };

    /**
     * @param {String} client_key
     * @return module:model/payment/Payment
     */
    exports.prototype.forClient = function (client_key) {
        var _this = this;
        Base.prototype.forClient.call(_this, client_key);
        return _this;
    };

    /**
     * Callback function to receive the result of the operation.
     * @callback module:model/payment/Payment~callApiCallback
     * @param {module:exception/OrbipayApiException} exception Error Response, if any.
     * @param {module:model/payment/Payment} data The data returned by the service call.
     */


    /**
     * @param {module:model/base/InvocationContext} invocation_context
     * @param {module:model/payment/Payment~callApiCallback} callback
     * @param {String} invocation_mode (optional). Use {module:enum/InvocationModeEnum} to get values.
     * @param {String} api_end_point (optional). In case of 'InvocationModeEnum.test', For example http://localhost:8080/payments/v1
     * @return {Object} response consisting of 'exception' in case of validation failure other wise 'request', The SuperAgent request object.
     */
    exports.prototype.create = function (invocation_context, callback, invocation_mode, api_end_point) {

        var handler;
        var _this = this;

        var internalCallback = function (errorMessage, parsedData, httpResponse, exception) {
            try {
                var response = ResponseWrapper.wrapResponse(exports, exception, errorMessage, parsedData, httpResponse, _this);
                if (callback) {
                    callback(response['exception'], response['data'])
                }
            } catch (e) {
                exception = OrbipayApiException.getDefaultException(e, _this);
                if (callback) {
                    callback(exception)
                }
            }
        };

        _this.withContext(invocation_context, invocation_mode, api_end_point);
        handler = new CreatePaymentHandler();

        if (callback && ValidationUtil.isFunction(callback)) {
            return handler.handleRequest(_this, internalCallback);
        } else {
            return handler.handleRequest(_this);
        }
    };

    /**
     * @param {module:model/base/InvocationContext} invocation_context
     * @param {module:model/payment/Payment~callApiCallback} callback
     * @param {String} invocation_mode (optional). Use {module:enum/InvocationModeEnum} to get values.
     * @param {String} api_end_point (optional). In case of 'InvocationModeEnum.test', For example http://localhost:8080/payments/v1
     * @return {Object} response consisting of 'exception' in case of validation failure other wise 'request', The SuperAgent request object.
     */
    exports.prototype.get = function (invocation_context, callback, invocation_mode, api_end_point) {

        var handler;
        var _this = this;

        var internalCallback = function (errorMessage, parsedData, httpResponse, exception) {
            try {
                var response = ResponseWrapper.wrapResponse(exports, exception, errorMessage, parsedData, httpResponse, _this);
                if (callback) {
                    callback(response['exception'], response['data'])
                }
            } catch (e) {
                exception = OrbipayApiException.getDefaultException(e, _this);
                if (callback) {
                    callback(exception)
                }
            }
        };

        _this.withContext(invocation_context, invocation_mode, api_end_point);
        handler = new GetPaymentHandler();

        if (callback && ValidationUtil.isFunction(callback)) {
            return handler.handleRequest(_this, internalCallback);
        } else {
            return handler.handleRequest(_this);
        }
    };

    /**
     * @param {module:model/base/InvocationContext} invocation_context
     * @param {module:model/payment/Payment~callApiCallback} callback
     * @param {String} invocation_mode (optional). Use {module:enum/InvocationModeEnum} to get values.
     * @param {String} api_end_point (optional). In case of 'InvocationModeEnum.test', For example http://localhost:8080/payments/v1
     * @return {Object} response consisting of 'exception' in case of validation failure other wise 'request', The SuperAgent request object.
     */
    exports.prototype.update = function (invocation_context, callback, invocation_mode, api_end_point) {

        var handler;
        var _this = this;

        var internalCallback = function (errorMessage, parsedData, httpResponse, exception) {
            try {
                var response = ResponseWrapper.wrapResponse(exports, exception, errorMessage, parsedData, httpResponse, _this);
                if (callback) {
                    callback(response['exception'], response['data'])
                }
            } catch (e) {
                exception = OrbipayApiException.getDefaultException(e, _this);
                if (callback) {
                    callback(exception)
                }
            }
        };

        _this.withContext(invocation_context, invocation_mode, api_end_point);
        handler = new UpdatePaymentHandler();

        if (callback && ValidationUtil.isFunction(callback)) {
            return handler.handleRequest(_this, internalCallback);
        } else {
            return handler.handleRequest(_this);
        }
    };

    /**
     * @param {module:model/base/InvocationContext} invocation_context
     * @param {module:model/payment/Payment~callApiCallback} callback
     * @param {String} invocation_mode (optional). Use {module:enum/InvocationModeEnum} to get values.
     * @param {String} api_end_point (optional). In case of 'InvocationModeEnum.test', For example http://localhost:8080/payments/v1
     * @return {Object} response consisting of 'exception' in case of validation failure other wise 'request', The SuperAgent request object.
     */
    exports.prototype.delete = function (invocation_context, callback, invocation_mode, api_end_point) {

        var handler;
        var _this = this;

        var internalCallback = function (errorMessage, parsedData, httpResponse, exception) {
            try {
                var response = ResponseWrapper.wrapResponse(exports, exception, errorMessage, parsedData, httpResponse, _this);
                if (callback) {
                    callback(response['exception'], response['data'])
                }
            } catch (e) {
                exception = OrbipayApiException.getDefaultException(e, _this);
                if (callback) {
                    callback(exception)
                }
            }
        };

        _this.withContext(invocation_context, invocation_mode, api_end_point);
        handler = new DeletePaymentHandler();

        if (callback && ValidationUtil.isFunction(callback)) {
            return handler.handleRequest(_this, internalCallback);
        } else {
            return handler.handleRequest(_this);
        }
    };

    return exports;
}));


