/**
 * Created by Alacriti Inc on 8/3/18.
 */
(function (root, factory) {
    if (typeof module === 'object' && module.exports) {
        // CommonJS-like environments that support module.exports, like Node.
        module.exports = factory(require('logtown').getLogger('orbipay-paymentsapi-client/model/refund/FeeRefund'));
    }
}(this, function (logger) {
    'use strict';


    /**
     * The Fee model module.
     * @module model/refund/FeeRefund
     * @version 1.0.0
     */

    /**
     * Constructs a new <code>Fee</code>.
     * @alias module:model/refund/FeeRefund
     * @class
     */
    var exports = function (fee_refund_amount) {
        var _this = this;
        _this['fee_refund_amount'] = fee_refund_amount;
    };

    /**
     * @private
     * Constructs a <code>Fee</code> from a plain JavaScript object, optionally creating a new instance.
     * Copies all relevant properties from <code>data</code> to <code>obj</code> if supplied or a new instance if not.
     * @param {Object} data The plain JavaScript object bearing properties of interest.
     * @param {module:model/refund/FeeRefund} obj Optional instance to populate.
     * @return {module:model/refund/FeeRefund} The populated <code>Fee</code> instance.
     */
    exports.constructFromObject = function (data, obj) {
        if (data) {
            obj = obj || new exports();
            if (data.hasOwnProperty('fee_type')) {
                obj['fee_type'] = data['fee_type'];
            }
            if (data.hasOwnProperty('fee_refund_amount')) {
                obj['fee_refund_amount'] = data['fee_refund_amount'];
            }
        }
        return obj;
    };

    /**
     * The way the fee is being charged to the customer. Fee could be added to the amount being paid, or, it could be charged separately by the biller. The fee_type would default to add_to_principal if the fee is not configured for the client in Orbipay.
     * @member {module:model/refund/FeeRefund.FeeTypeEnum} fee_type
     */
    exports.prototype['fee_type'] = undefined;
    /**
     * The fee amount upto 2 decimal places.
     * @member {String} fee_refund_amount
     */
    exports.prototype['fee_refund_amount'] = undefined;

    return exports;
}));


