/**
 * Created by Alacriti Inc on 8/3/18.
 */
(function (root, factory) {
    if (typeof module === 'object' && module.exports) {
        // CommonJS-like environments that support module.exports, like Node.
        module.exports = factory(require('logtown').getLogger('orbipay-paymentsapi-client/util/ValidationUtil'), require('./../constant/HttpParamConstants'), require('./../constant/AppErrorCodes'), require('./../enum/ACHEligibleFlagEnum'), require('./../enum/ATMEligibleFlagEnum'), require('./../enum/CustomerAccountFilterEnum'), require('./../enum/CustomerAccountStatusEnum'), require('./../enum/CustomerFilterEnum'), require('./../enum/CustomerStatusEnum'), require('./../enum/DataTypeEnum'), require('./../enum/FeeTypeEnum'), require('./../enum/FundingAccountFilterEnum'), require('./../enum/FundingAccountHolderTypeEnum'), require('./../enum/FundingAccountStatusEnum'), require('./../enum/FundingAccountSubtypeEnum'), require('./../enum/FundingAccountTypeEnum'), require('./../enum/GenderEnum'), require('./../enum/InvocationModeEnum'), require('./../enum/MandatoryEnum'), require('./../enum/PaymentAmountTypeEnum'), require('./../enum/PaymentFilterEnum'), require('./../enum/PaymentMethodEnum'), require('./../enum/PaymentRecurringTypeEnum'), require('./../enum/PaymentScheduleTypeEnum'), require('./../enum/PaymentSetupFilterEnum'), require('./../enum/PaymentSetupScheduleTypeEnum'), require('./../enum/PaymentSetupStatusEnum'), require('./../enum/PaymentStatusEnum'), require('./../enum/ProductEnum'), require('./../enum/RefundStatusEnum'), require('./../enum/RefundTypeEnum'), require('./../enum/RequestorTypeEnum')
        );
    }
}(this, function (logger, HttpParamConstants, AppErrorCodes, ACHEligibleFlagEnum, ATMEligibleFlagEnum, CustomerAccountFilterEnum, CustomerAccountStatusEnum, CustomerFilterEnum, CustomerStatusEnum, DataTypeEnum, FeeTypeEnum, FundingAccountFilterEnum, FundingAccountHolderTypeEnum, FundingAccountStatusEnum, FundingAccountSubtypeEnum, FundingAccountTypeEnum, GenderEnum, InvocationModeEnum, MandatoryEnum, PaymentAmountTypeEnum, PaymentFilterEnum, PaymentMethodEnum, PaymentRecurringTypeEnum, PaymentScheduleTypeEnum, PaymentSetupFilterEnum, PaymentSetupScheduleTypeEnum, PaymentSetupStatusEnum, PaymentStatusEnum, ProductEnum, RefundStatusEnum, RefundTypeEnum, RequestorTypeEnum) {
        'use strict';


        /**
         * The ValidationUtil model module.
         * @module util/ValidationUtil
         * @version 1.0.0
         */

        /**
         * Constructs a new <code>ValidationUtil</code>.
         * @alias module:util/ValidationUtil
         * @class
         */
        var exports = function () {
        };

        exports.isRegExp = function (data) {
            return data && typeof data === 'object' && data.constructor === RegExp;
        };

        exports.isFunction = function (data) {
            return data && {}.toString.call(data) === '[object Function]';
        };

        exports.isError = function (data) {
            return data instanceof Error && typeof data.message !== 'undefined';
        };

        exports.isString = function (data) {
            return (typeof data === 'string' || data instanceof String);
        };

        exports.isNumber = function (data) {
            return typeof data === 'number' && isFinite(data);
        };

        exports.isArray = function (data) {
            return data && typeof data === 'object' && data.constructor === Array;
        };

        exports.isBoolean = function (data) {
            return typeof data === 'boolean';
        };

        exports.isNonEmptyObject = function (data) {
            return data !== null && typeof data === 'object';
        };

        exports.isNoneOrString = function (data) {
            return (data === undefined || data === null || exports.isString(data));
        };

        exports.isNoneOrObject = function (data) {
            return (data === undefined || data === null || exports.isNonEmptyObject(data));
        };

        exports.isNoneOrArray = function (data) {
            return (data === undefined || data === null || exports.isArray(data));
        };

        exports.isNonEmptyString = function (data) {
            return (exports.isString(data) && data !== '');
        };

        exports.validate = function (data, type, isMandatory, exception, fieldName, appErrorCode, validateFunction) {
            if (DataTypeEnum.String === type) {
                exports.validateString(data, isMandatory, exception, fieldName, appErrorCode, validateFunction);
            } else if (DataTypeEnum.Object === type) {
                exports.validateObject(data, isMandatory, exception, fieldName, appErrorCode, validateFunction);
            } else if (DataTypeEnum.Array === type) {
                exports.validateArray(data, isMandatory, exception, fieldName, appErrorCode, validateFunction);
            }
        };

        exports.validateString = function (data, isMandatory, exception, fieldName, appErrorCode, validateFunction) {
            if (!exports.isNoneOrString(data)) {
                exception.addFieldErrorAndResponseCodes(fieldName, appErrorCode);
                return;
            }

            if (MandatoryEnum.yes === isMandatory) {
                exports.validateMandatoryString(data, exception, fieldName, appErrorCode, validateFunction)
            } else {
                exports.validateOptionalString(data, exception, fieldName, appErrorCode, validateFunction)
            }
        };

        exports.validateObject = function (data, isMandatory, exception, fieldName, appErrorCode, validateFunction) {
            if (!exports.isNoneOrObject(data)) {
                exception.addFieldErrorAndResponseCodes(fieldName, appErrorCode);
                return;
            }

            if (MandatoryEnum.yes === isMandatory) {
                exports.validateMandatoryObject(data, exception, fieldName, appErrorCode, validateFunction);
            } else {
                exports.validateOptionalObject(data, exception, fieldName, appErrorCode, validateFunction);
            }
        };

        exports.validateArray = function (data, isMandatory, exception, fieldName, appErrorCode, validateFunction) {
            if (!exports.isNoneOrArray(data)) {
                exception.addFieldErrorAndResponseCodes(fieldName, appErrorCode);
                return;
            }

            if (MandatoryEnum.yes === isMandatory) {
                exports.validateMandatoryArray(data, exception, fieldName, appErrorCode, validateFunction);
            } else {
                exports.validateOptionalArray(data, exception, fieldName, appErrorCode, validateFunction);
            }
        };


        exports.validateMandatoryString = function (data, exception, fieldName, appErrorCode, validateFunction) {
            if (!data || (validateFunction && !validateFunction(data))) {
                exception.addFieldErrorAndResponseCodes(fieldName, appErrorCode);
            }
        };

        exports.validateOptionalString = function (data, exception, fieldName, appErrorCode, validateFunction) {
            if (data && (validateFunction && !validateFunction(data))) {
                exception.addFieldErrorAndResponseCodes(fieldName, appErrorCode);
            }
        };

        exports.validateMandatoryObject = function (data, exception, fieldName, appErrorCode, validateFunction) {
            if (!data) {
                exception.addFieldErrorAndResponseCodes(fieldName, appErrorCode);
                return;
            }

            if (validateFunction) {
                validateFunction(data, exception, fieldName)
            }
        };

        exports.validateOptionalObject = function (data, exception, fieldName, appErrorCode, validateFunction) {
            if (data && (validateFunction )) {
                validateFunction(data, exception, fieldName);
            }
        };


        exports.validateMandatoryArray = function (data, exception, fieldName, appErrorCode, validateFunction) {
            if (!data) {
                exception.addFieldErrorAndResponseCodes(fieldName, appErrorCode);
                return;
            }

            if (validateFunction) {
                validateFunction(data, exception, fieldName)
            }
        };

        exports.validateOptionalArray = function (data, exception, fieldName, appErrorCode, validateFunction) {
            if (data && (validateFunction )) {
                validateFunction(data, exception, fieldName);
            }
        };

        exports.isValidName = function (data) {
            return /^[0-9a-zA-Z #,.'&/\-@!$%*()_+={}|:;`\[\]^~\\"]{0,60}$/.test(data);
        };

        exports.isValidDateOfBirth = function (data) {
            return exports.isValidDate(data) && !exports.isFutureDate(data);
        };

        exports.isValidLocale = function (data) {
            return /^[a-z]{2}_[A-Z]{2}$/.test(data);
        };

        exports.isValidSSN = function (data) {
            return /^[0-9]{9}$/.test(data);
        };

        exports.isValidEmail = function (data) {
            return data.length <= 64 && /[a-zA-Z0-9!#$%&'*+-\/=?^._`{|}~]+@[a-zA-Z0-9_.-]+\.[a-zA-Z0-9._]+/.test(data);
        };

        exports.isValidCustomerReference = function (data) {
            return /^[0-9a-zA-Z #,.'&\/\-@!$%*()_+={}|:;`\[\]^~\s\\"]{0,100}$/.test(data);
        };

        exports.isValidComments = function (data) {
            return /^[0-9a-zA-Z #,.'&\/\-@!$%*()_+={}|:;`\[\]^~\s\\"]{0,1000}$/.test(data);
        };

        exports.isValidAddressLine = function (data) {
            return /^[0-9a-zA-Z #,.'&\/\-@!$%*()_+={}|:;`\[\]^~\\"]{0,100}$/.test(data);
        };

        exports.isValidCity = function (data) {
            return /^[0-9a-zA-Z ,.-]{0,20}$/.test(data);
        };

        exports.isValidState = function (data) {
            return /^[a-zA-Z]{2}$/.test(data);
        };

        exports.isValidCountry = function (data) {
            return /^[a-zA-Z]{3}$/.test(data);
        };

        exports.isValidPhone = function (data) {
            return /^[0-9]{10}$/.test(data);
        };

        exports.isValidZip1 = function (data) {
            return /^[a-zA-Z0-9 -]{2,10}$/.test(data);
        };

        exports.isValidZip2 = function (data) {
            return /^[0-9]{4}$/.test(data);
        };

        exports.isValidDate = function (data) {
            return /^[0-9]{4}-(0[1-9]|1[0-2])-(0[1-9]|[1-2][0-9]|3[0-1])$/.test(data);
        };

        exports.isValidCustomerAccountBalance = function (data) {
            return /^([+-])?([0-9]{1,14}(\.[0-9][0-9]?)?)$/.test(data);
        };

        exports.isValidAmount = function (data) {
            return /^([0-9]{1,14}(\.[0-9][0-9]?)?)$/.test(data);
        };

        exports.isValidCustomerAccountReference = function (data) {
            return /^[a-zA-Z0-9._-]{0,50}$/.test(data);
        };

        exports.isValidNickName = function (data) {
            return /^[0-9a-zA-Z #,.'&\/\-@!$%*()_+={}|:;`\[\]^~\\"]{0,50}$/.test(data);
        };

        exports.isValidCustomerAccountNumber = function (data) {
            return /^[a-zA-Z0-9-_\/!@#$%&*{}.]{1,32}$/.test(data);
        };

        exports.isValidAccountHolderName = function (data) {
            return exports.isValidName(data);
        };

        exports.isValidFundingAccountNumber = function (data) {
            return /^[a-zA-Z0-9]{4,32}$/.test(data);
        };

        exports.isValidExpiryDate = function (data) {
            return /^[0-9]{2}\/[0-9]{2}$/.test(data);
        };

        exports.isValidCardCVVNumber = function (data) {
            return /^[0-9]{3,4}$/.test(data);
        };

        exports.isValidPaymentReference = function (data) {
            return /^[a-zA-Z0-9._-]{0,50}$/.test(data);
        };

        exports.isValidPaymentSetupReference = function (data) {
            return /^[a-zA-Z0-9._-]{0,50}$/.test(data);
        };

        exports.isValidPaymentRecurringCount = function (data) {
            return /^[0-9]{1,3}$/.test(data);
        };

        exports.isValidId = function (data) {
            return /^[0-9]{0,20}$/.test(data);
        };

        exports.isValidConfirmationNumber = function (data) {
            return /^[a-zA-Z0-9]{0,30}$/.test(data);
        };

        exports.isValidPageSize = function (data) {
            if (/^[0-9]{0,3}$/.test(data)) {
                var integerValue = parseInt(data);
                if (1 <= integerValue && integerValue <= 100) {
                    return true;
                }
            }
            return false;
        };

        exports.isValidApiEndPoint = function (data) {
            return /^(?:http(s)?:\/\/)?[\w.-]+(?:\.[\w\.-]+)+[\w\-\._~:/?#[\]@!\$&'\(\)\*\+,;=]+$/.test(data);
        };

        exports.isValidFromIndex = function (data) {
            if (/^[0-9]{0,4}$/.test(data)) {
                var integerValue = parseInt(data);
                if (1 <= integerValue && integerValue <= 5000) {
                    return true;
                }
            }
            return false;
        };

        exports.isValidToIndex = function (data) {
            if (/^[0-9]{0,4}$/.test(data)) {
                var integerValue = parseInt(data);
                if (1 <= integerValue && integerValue <= 5000) {
                    return true;
                }
            }
            return false;
        };

        exports.isValidQueryId = function (data) {
            return /^[a-zA-Z0-9]{0,50}$/.test(data);
        };

        exports.isValidPaymentPlanId = function (data) {
            return /^[0-9]{0,9}$/.test(data);
        };

        exports.isValidPaymentDeferDays = function (data) {
            return /^[0-9]{1,2}$/.test(data);
        };

        exports.isValidRefundReference = function (data) {
            return /^[a-zA-Z0-9._-]{0,50}$/.test(data);
        };

        exports.isValidChannel = function (data) {
            return data.length <= 100;
        };

        exports.isValidClientKey = function (data) {
            return /^[a-zA-Z0-9_-]{1,50}$/.test(data);
        };

        exports.isValidIdempotentRequestKey = function (data) {
            return /^[a-zA-Z0-9]{1,50}$/.test(data);
        };

        exports.isValidRequestor = function (data) {
            return /^[a-zA-Z0-9]{1,50}$/.test(data);
        };

        exports.isValidABARoutingNumber = function (routing) {
            if (!/^[0-9]{9}$/.test(routing)) {
                return false;
            }
            // http://en.wikipedia.org/wiki/Routing_transit_number#MICR_Routing_number_format
            var checksumTotal = (7 * (parseInt(routing.charAt(0), 10) + parseInt(routing.charAt(3), 10) + parseInt(routing.charAt(6), 10))) +
                (3 * (parseInt(routing.charAt(1), 10) + parseInt(routing.charAt(4), 10) + parseInt(routing.charAt(7), 10))) +
                (9 * (parseInt(routing.charAt(2), 10) + parseInt(routing.charAt(5), 10) + parseInt(routing.charAt(8), 10)));

            var checksumMod = checksumTotal % 10;

            return checksumMod === 0;
        };

        exports.isFutureDate = function (data) {
            var parts = data.split('-');
            // Please pay attention to the month (parts[1]); JavaScript counts months from 0:
            // January - 0, February - 1, etc.
            var inputDate = new Date(parts[0], parts[1] - 1, parts[2]);
            var currentDate = new Date();

            return (inputDate > currentDate);
        };

        exports.isXSSText = function (data) {
            if (!exports.isNonEmptyString(data)) {
                return false;
            }
            return /<(|\/|[^\/>][^>]+|\/[^>][^>]+)>/.test(data);
        };

        exports.isValidACHEligibleFlag = function (data) {
            for (var key in ACHEligibleFlagEnum) {
                if (ACHEligibleFlagEnum.hasOwnProperty(key) && ACHEligibleFlagEnum[key] === data) {
                    return true;
                }
            }
            return false;
        };

        exports.isValidATMEligibleFlag = function (data) {
            for (var key in ATMEligibleFlagEnum) {
                if (ATMEligibleFlagEnum.hasOwnProperty(key) && ATMEligibleFlagEnum[key] === data) {
                    return true;
                }
            }
            return false;
        };

        exports.isValidCustomerAccountFilter = function (data) {
            for (var key in CustomerAccountFilterEnum) {
                if (CustomerAccountFilterEnum.hasOwnProperty(key) && CustomerAccountFilterEnum[key] === data) {
                    return true;
                }
            }
            return false;
        };

        exports.isValidCustomerAccountStatus = function (data) {
            for (var key in CustomerAccountStatusEnum) {
                if (CustomerAccountStatusEnum.hasOwnProperty(key) && CustomerAccountStatusEnum[key] === data) {
                    return true;
                }
            }
            return false;
        };

        exports.isValidCustomerAccountStatuses = function (data, exception, fieldName) {
            for (var key in data) {
                if (data.hasOwnProperty(key) && !exports.isValidCustomerAccountStatus(data[key])) {
                    exception.addFieldErrorAndResponseCodes(fieldName, AppErrorCodes.CUSTOMER_ACCOUNT_STATUS_INVALID);
                    return false;
                }
            }
            return true;
        };

        exports.isValidCustomerFilter = function (data) {
            for (var key in CustomerFilterEnum) {
                if (CustomerFilterEnum.hasOwnProperty(key) && CustomerFilterEnum[key] === data) {
                    return true;
                }
            }
            return false;
        };

        exports.isValidCustomerStatus = function (data) {
            for (var key in CustomerStatusEnum) {
                if (CustomerStatusEnum.hasOwnProperty(key) && CustomerStatusEnum[key] === data) {
                    return true;
                }
            }
            return false;
        };

        exports.isValidCustomerStatuses = function (data, exception, fieldName) {
            for (var key in data) {
                if (data.hasOwnProperty(key) && !exports.isValidCustomerStatus(data[key])) {
                    exception.addFieldErrorAndResponseCodes(fieldName, AppErrorCodes.CUSTOMER_STATUS_INVALID);
                    return false;
                }
            }
            return true;
        };

        exports.isValidDataType = function (data) {
            for (var key in DataTypeEnum) {
                if (DataTypeEnum.hasOwnProperty(key) && DataTypeEnum[key] === data) {
                    return true;
                }
            }
            return false;
        };

        exports.isValidFeeType = function (data) {
            for (var key in FeeTypeEnum) {
                if (FeeTypeEnum.hasOwnProperty(key) && FeeTypeEnum[key] === data) {
                    return true;
                }
            }
            return false;
        };

        exports.isValidFundingAccountFilter = function (data) {
            for (var key in FundingAccountFilterEnum) {
                if (FundingAccountFilterEnum.hasOwnProperty(key) && FundingAccountFilterEnum[key] === data) {
                    return true;
                }
            }
            return false;
        };

        exports.isValidFundingAccountHolderType = function (data) {
            for (var key in FundingAccountHolderTypeEnum) {
                if (FundingAccountHolderTypeEnum.hasOwnProperty(key) && FundingAccountHolderTypeEnum[key] === data) {
                    return true;
                }
            }
            return false;
        };

        exports.isValidFundingAccountStatus = function (data) {
            for (var key in FundingAccountStatusEnum) {
                if (FundingAccountStatusEnum.hasOwnProperty(key) && FundingAccountStatusEnum[key] === data) {
                    return true;
                }
            }
            return false;
        };

        exports.isValidFundingAccountStatuses = function (data, exception, fieldName) {
            for (var key in data) {
                if (data.hasOwnProperty(key) && !exports.isValidFundingAccountStatus(data[key])) {
                    exception.addFieldErrorAndResponseCodes(fieldName, AppErrorCodes.FUNDING_ACCOUNT_STATUS_INVALID);
                    return false;
                }
            }
            return true;
        };

        exports.isValidFundingAccountSubtype = function (data) {
            for (var key in FundingAccountSubtypeEnum) {
                if (FundingAccountSubtypeEnum.hasOwnProperty(key) && FundingAccountSubtypeEnum[key] === data) {
                    return true;
                }
            }
            return false;
        };

        exports.isValidFundingAccountSubtypes = function (data, exception, fieldName) {
            for (var key in data) {
                if (data.hasOwnProperty(key) && !exports.isValidFundingAccountSubtype(data[key])) {
                    exception.addFieldErrorAndResponseCodes(fieldName, AppErrorCodes.FUNDING_ACCOUNT_SUBTYPE_INVALID);
                    return false;
                }
            }
            return true;
        };

        exports.isValidFundingAccountType = function (data) {
            for (var key in FundingAccountTypeEnum) {
                if (FundingAccountTypeEnum.hasOwnProperty(key) && FundingAccountTypeEnum[key] === data) {
                    return true;
                }
            }
            return false;
        };

        exports.isValidFundingAccountTypes = function (data, exception, fieldName) {
            for (var key in data) {
                if (data.hasOwnProperty(key) && !exports.isValidFundingAccountType(data[key])) {
                    exception.addFieldErrorAndResponseCodes(fieldName, AppErrorCodes.FORM_PARAM_FUNDING_ACCOUNT_TYPE_INVALID);
                    return false;
                }
            }
            return true;
        };

        exports.isValidGender = function (data) {
            for (var key in GenderEnum) {
                if (GenderEnum.hasOwnProperty(key) && GenderEnum[key] === data) {
                    return true;
                }
            }
            return false;
        };

        exports.isValidInvocationMode = function (data) {
            for (var key in InvocationModeEnum) {
                if (InvocationModeEnum.hasOwnProperty(key) && InvocationModeEnum[key] === data) {
                    return true;
                }
            }
            return false;
        };

        exports.isValidPaymentAmountType = function (data) {
            for (var key in PaymentAmountTypeEnum) {
                if (PaymentAmountTypeEnum.hasOwnProperty(key) && PaymentAmountTypeEnum[key] === data) {
                    return true;
                }
            }
            return false;
        };

        exports.isValidPaymentFilter = function (data) {
            for (var key in PaymentFilterEnum) {
                if (PaymentFilterEnum.hasOwnProperty(key) && PaymentFilterEnum[key] === data) {
                    return true;
                }
            }
            return false;
        };

        exports.isValidPaymentMethod = function (data) {
            for (var key in PaymentMethodEnum) {
                if (PaymentMethodEnum.hasOwnProperty(key) && PaymentMethodEnum[key] === data) {
                    return true;
                }
            }
            return false;
        };

        exports.isValidPaymentRecurringType = function (data) {
            for (var key in PaymentRecurringTypeEnum) {
                if (PaymentRecurringTypeEnum.hasOwnProperty(key) && PaymentRecurringTypeEnum[key] === data) {
                    return true;
                }
            }
            return false;
        };

        exports.isValidPaymentScheduleType = function (data) {
            for (var key in PaymentScheduleTypeEnum) {
                if (PaymentScheduleTypeEnum.hasOwnProperty(key) && PaymentScheduleTypeEnum[key] === data) {
                    return true;
                }
            }
            return false;
        };

        exports.isValidPaymentScheduleTypes = function (data, exception, fieldName) {
            for (var key in data) {
                if (data.hasOwnProperty(key) && !exports.isValidPaymentScheduleType(data[key])) {
                    exception.addFieldErrorAndResponseCodes(fieldName, AppErrorCodes.PAYMENT_SCHEDULE_TYPE_INVALID);
                    return false;
                }
            }
            return true;
        };

        exports.isValidPaymentSetupFilter = function (data) {
            for (var key in PaymentSetupFilterEnum) {
                if (PaymentSetupFilterEnum.hasOwnProperty(key) && PaymentSetupFilterEnum[key] === data) {
                    return true;
                }
            }
            return false;
        };

        exports.isValidPaymentSetupScheduleType = function (data) {
            for (var key in PaymentSetupScheduleTypeEnum) {
                if (PaymentSetupScheduleTypeEnum.hasOwnProperty(key) && PaymentSetupScheduleTypeEnum[key] === data) {
                    return true;
                }
            }
            return false;
        };

        exports.isValidPaymentSetupScheduleTypes = function (data, exception, fieldName) {
            for (var key in data) {
                if (data.hasOwnProperty(key) && !exports.isValidPaymentSetupScheduleType(data[key])) {
                    exception.addFieldErrorAndResponseCodes(fieldName, AppErrorCodes.PAYMENT_SETUP_SCHEDULE_TYPE_INVALID);
                    return false;
                }
            }
            return true;
        };

        exports.isValidPaymentSetupStatus = function (data) {
            for (var key in PaymentSetupStatusEnum) {
                if (PaymentSetupStatusEnum.hasOwnProperty(key) && PaymentSetupStatusEnum[key] === data) {
                    return true;
                }
            }
            return false;
        };

        exports.isValidPaymentSetupStatuses = function (data, exception, fieldName) {
            for (var key in data) {
                if (data.hasOwnProperty(key) && !exports.isValidPaymentSetupStatus(data[key])) {
                    exception.addFieldErrorAndResponseCodes(fieldName, AppErrorCodes.QUERY_PARAM_PAYMENT_SETUP_STATUS_INVALID);
                    return false;
                }
            }
            return true;
        };

        exports.isValidPaymentStatus = function (data) {
            for (var key in PaymentStatusEnum) {
                if (PaymentStatusEnum.hasOwnProperty(key) && PaymentStatusEnum[key] === data) {
                    return true;
                }
            }
            return false;
        };

        exports.isValidPaymentStatuses = function (data, exception, fieldName) {
            for (var key in data) {
                if (data.hasOwnProperty(key) && !exports.isValidPaymentStatus(data[key])) {
                    exception.addFieldErrorAndResponseCodes(fieldName, AppErrorCodes.QUERY_PARAM_PAYMENT_STATUS_INVALID);
                    return false;
                }
            }
            return true;
        };

        exports.isValidProduct = function (data) {
            for (var key in ProductEnum) {
                if (ProductEnum.hasOwnProperty(key) && ProductEnum[key] === data) {
                    return true;
                }
            }
            return false;
        };

        exports.isValidRefundStatus = function (data) {
            for (var key in RefundStatusEnum) {
                if (RefundStatusEnum.hasOwnProperty(key) && RefundStatusEnum[key] === data) {
                    return true;
                }
            }
            return false;
        };

        exports.isValidRefundStatuses = function (data, exception, fieldName) {
            for (var key in data) {
                if (data.hasOwnProperty(key) && !exports.isValidRefundStatus(data[key])) {
                    exception.addFieldErrorAndResponseCodes(fieldName, AppErrorCodes.REFUND_STATUS_INVALID);
                    return false;
                }
            }
            return true;
        };

        exports.isValidRefundType = function (data) {
            for (var key in RefundTypeEnum) {
                if (RefundTypeEnum.hasOwnProperty(key) && RefundTypeEnum[key] === data) {
                    return true;
                }
            }
            return false;
        };

        exports.isValidRefundTypes = function (data, exception, fieldName) {
            for (var key in data) {
                if (data.hasOwnProperty(key) && !exports.isValidRefundType(data[key])) {
                    exception.addFieldErrorAndResponseCodes(fieldName, AppErrorCodes.REFUND_TYPE_INVALID);
                    return false;
                }
            }
            return true;
        };

        exports.isValidRequestorType = function (data) {
            for (var key in RequestorTypeEnum) {
                if (RequestorTypeEnum.hasOwnProperty(key) && RequestorTypeEnum[key] === data) {
                    return true;
                }
            }
            return false;
        };

        exports.validateCustomerAddress = function (address, exception, segment) {
            segment += HttpParamConstants.DOT;
            exports.validate(address[HttpParamConstants.ADDRESS_LINE1], DataTypeEnum.String, MandatoryEnum.no, exception, segment + HttpParamConstants.ADDRESS_LINE1, AppErrorCodes.CUSTOMER_ADDRESS_LINE1_INVALID, exports.isValidAddressLine);
            exports.validate(address[HttpParamConstants.ADDRESS_LINE2], DataTypeEnum.String, MandatoryEnum.no, exception, segment + HttpParamConstants.ADDRESS_LINE2, AppErrorCodes.CUSTOMER_ADDRESS_LINE2_INVALID, exports.isValidAddressLine);
            exports.validate(address[HttpParamConstants.ADDRESS_CITY], DataTypeEnum.String, MandatoryEnum.no, exception, segment + HttpParamConstants.ADDRESS_CITY, AppErrorCodes.CUSTOMER_ADDRESS_CITY_INVALID, exports.isValidCity);
            exports.validate(address[HttpParamConstants.ADDRESS_STATE], DataTypeEnum.String, MandatoryEnum.no, exception, segment + HttpParamConstants.ADDRESS_STATE, AppErrorCodes.CUSTOMER_ADDRESS_STATE_INVALID, exports.isValidState);
            exports.validate(address[HttpParamConstants.ADDRESS_COUNTRY], DataTypeEnum.String, MandatoryEnum.no, exception, segment + HttpParamConstants.ADDRESS_COUNTRY, AppErrorCodes.CUSTOMER_ADDRESS_COUNTRY_INVALID, exports.isValidCountry);
            exports.validate(address[HttpParamConstants.ADDRESS_ZIP1], DataTypeEnum.String, MandatoryEnum.no, exception, segment + HttpParamConstants.ADDRESS_ZIP1, AppErrorCodes.CUSTOMER_ADDRESS_ZIP1_INVALID, exports.isValidZip1);
            exports.validate(address[HttpParamConstants.ADDRESS_ZIP2], DataTypeEnum.String, MandatoryEnum.no, exception, segment + HttpParamConstants.ADDRESS_ZIP2, AppErrorCodes.CUSTOMER_ADDRESS_ZIP2_INVALID, exports.isValidZip2);
        };

        exports.validateCustomerAccountAddress = function (address, exception, segment) {
            segment += HttpParamConstants.DOT;
            exports.validate(address[HttpParamConstants.ADDRESS_LINE1], DataTypeEnum.String, MandatoryEnum.no, exception, segment + HttpParamConstants.ADDRESS_LINE1, AppErrorCodes.CUSTOMER_ACCOUNT_ADDRESS_LINE1_INVALID, exports.isValidAddressLine);
            exports.validate(address[HttpParamConstants.ADDRESS_LINE2], DataTypeEnum.String, MandatoryEnum.no, exception, segment + HttpParamConstants.ADDRESS_LINE2, AppErrorCodes.CUSTOMER_ACCOUNT_ADDRESS_LINE2_INVALID, exports.isValidAddressLine);
            exports.validate(address[HttpParamConstants.ADDRESS_CITY], DataTypeEnum.String, MandatoryEnum.no, exception, segment + HttpParamConstants.ADDRESS_CITY, AppErrorCodes.CUSTOMER_ACCOUNT_ADDRESS_CITY_INVALID, exports.isValidCity);
            exports.validate(address[HttpParamConstants.ADDRESS_STATE], DataTypeEnum.String, MandatoryEnum.no, exception, segment + HttpParamConstants.ADDRESS_STATE, AppErrorCodes.CUSTOMER_ACCOUNT_ADDRESS_STATE_INVALID, exports.isValidState);
            exports.validate(address[HttpParamConstants.ADDRESS_COUNTRY], DataTypeEnum.String, MandatoryEnum.no, exception, segment + HttpParamConstants.ADDRESS_COUNTRY, AppErrorCodes.CUSTOMER_ACCOUNT_ADDRESS_COUNTRY_INVALID, exports.isValidCountry);
            exports.validate(address[HttpParamConstants.ADDRESS_ZIP1], DataTypeEnum.String, MandatoryEnum.no, exception, segment + HttpParamConstants.ADDRESS_ZIP1, AppErrorCodes.CUSTOMER_ACCOUNT_ADDRESS_ZIP1_INVALID, exports.isValidZip1);
            exports.validate(address[HttpParamConstants.ADDRESS_ZIP2], DataTypeEnum.String, MandatoryEnum.no, exception, segment + HttpParamConstants.ADDRESS_ZIP2, AppErrorCodes.CUSTOMER_ACCOUNT_ADDRESS_ZIP2_INVALID, exports.isValidZip2);
        };

        exports.validateFundingAccountAddress = function (address, exception, segment) {
            segment += HttpParamConstants.DOT;
            exports.validate(address[HttpParamConstants.ADDRESS_LINE1], DataTypeEnum.String, MandatoryEnum.no, exception, segment + HttpParamConstants.ADDRESS_LINE1, AppErrorCodes.FUNDING_ACCOUNT_ADDRESS_LINE1_INVALID, exports.isValidAddressLine);
            exports.validate(address[HttpParamConstants.ADDRESS_LINE2], DataTypeEnum.String, MandatoryEnum.no, exception, segment + HttpParamConstants.ADDRESS_LINE2, AppErrorCodes.FUNDING_ACCOUNT_ADDRESS_LINE2_INVALID, exports.isValidAddressLine);
            exports.validate(address[HttpParamConstants.ADDRESS_CITY], DataTypeEnum.String, MandatoryEnum.no, exception, segment + HttpParamConstants.ADDRESS_CITY, AppErrorCodes.FUNDING_ACCOUNT_ADDRESS_CITY_INVALID, exports.isValidCity);
            exports.validate(address[HttpParamConstants.ADDRESS_STATE], DataTypeEnum.String, MandatoryEnum.no, exception, segment + HttpParamConstants.ADDRESS_STATE, AppErrorCodes.FUNDING_ACCOUNT_ADDRESS_STATE_INVALID, exports.isValidState);
            exports.validate(address[HttpParamConstants.ADDRESS_COUNTRY], DataTypeEnum.String, MandatoryEnum.no, exception, segment + HttpParamConstants.ADDRESS_COUNTRY, AppErrorCodes.FUNDING_ACCOUNT_ADDRESS_COUNTRY_INVALID, exports.isValidCountry);
            exports.validate(address[HttpParamConstants.ADDRESS_ZIP1], DataTypeEnum.String, MandatoryEnum.no, exception, segment + HttpParamConstants.ADDRESS_ZIP1, AppErrorCodes.FUNDING_ACCOUNT_ADDRESS_ZIP1_INVALID, exports.isValidZip1);
            exports.validate(address[HttpParamConstants.ADDRESS_ZIP2], DataTypeEnum.String, MandatoryEnum.no, exception, segment + HttpParamConstants.ADDRESS_ZIP2, AppErrorCodes.FUNDING_ACCOUNT_ADDRESS_ZIP2_INVALID, exports.isValidZip2);
        };

        exports.validateInvocationContext = function (invocationContext, exception) {
            exports.validate(invocationContext[HttpParamConstants.CHANNEL], DataTypeEnum.String, MandatoryEnum.yes, exception, HttpParamConstants.CHANNEL, AppErrorCodes.CHANNEL_INVALID, exports.isValidChannel);
            exports.validate(invocationContext[HttpParamConstants.CLIENT_KEY], DataTypeEnum.String, MandatoryEnum.yes, exception, HttpParamConstants.CLIENT_KEY, AppErrorCodes.CLIENT_KEY_INVALID, exports.isValidClientKey);
            exports.validate(invocationContext[HttpParamConstants.IDEMPOTENT_REQUEST_KEY], DataTypeEnum.String, MandatoryEnum.yes, exception, HttpParamConstants.IDEMPOTENT_REQUEST_KEY, AppErrorCodes.IDEMPOTENT_REQUEST_KEY_INVALID, exports.isValidIdempotentRequestKey);
            exports.validate(invocationContext[HttpParamConstants.REQUESTOR], DataTypeEnum.String, MandatoryEnum.yes, exception, HttpParamConstants.REQUESTOR, AppErrorCodes.REQUESTOR_INVALID, exports.isValidRequestor);
            exports.validate(invocationContext[HttpParamConstants.REQUESTOR_TYPE], DataTypeEnum.String, MandatoryEnum.yes, exception, HttpParamConstants.REQUESTOR_TYPE, AppErrorCodes.REQUESTOR_TYPE_INVALID, exports.isValidRequestorType);
            exports.validate(invocationContext[HttpParamConstants.SECRET], DataTypeEnum.String, MandatoryEnum.yes, exception, HttpParamConstants.SECRET, AppErrorCodes.secret_INVALID);
            exports.validate(invocationContext[HttpParamConstants.ATTRIBUTES], DataTypeEnum.Object, MandatoryEnum.yes, exception, HttpParamConstants.ATTRIBUTES, AppErrorCodes.ATTRIBUTES_INVALID, exports.validateAttributes);
        };

        exports.validateAttributes = function (attributes, exception, segment) {
            var list = attributes[HttpParamConstants.LIST];
            segment += HttpParamConstants.DOT + HttpParamConstants.LIST + HttpParamConstants.DOT;
            var keys = [];
            if (Array.isArray(list)) {
                for (var i = 0; i < list.length; i++) {

                    var attribute = list[i];
                    var key = attribute['key'];
                    var value = attribute['value'];

                    if (!exports.isNoneOrString(key) || !exports.isNonEmptyString(key) || keys.indexOf(key) > -1) {
                        exception.addFieldErrorAndResponseCodes(segment + key, AppErrorCodes.ATTRIBUTES_INVALID);
                    } else {
                        if (!exports.isNoneOrString(value) || !exports.isNonEmptyString(value)) {
                            exception.addFieldErrorAndResponseCodes(segment + key, AppErrorCodes.ATTRIBUTES_INVALID);
                        }
                    }
                    keys.push(key);
                }
            }
        };

        /**
         * @param {Object} customFields Optional instance to populate.
         * @param {String} segment Optional instance to populate.
         * @param {module:exception/OrbipayApiException} exception Optional instance to populate.
         */
        exports.validateCustomFields = function (customFields, exception, segment) {
            segment += HttpParamConstants.DOT;
            for (var key in customFields) {
                if (customFields.hasOwnProperty(key)) {
                    var value = customFields[key];
                    if (!exports.isNonEmptyString(key)) {
                        exception.addFieldErrorAndResponseCodes(segment + key, AppErrorCodes.CUSTOM_FIELD_KEY_INVALID)
                    } else {
                        if (!exports.isString(value)) {
                            exception.addFieldErrorAndResponseCodes(segment + key, AppErrorCodes.CUSTOM_FIELD_VALUE_INVALID)
                        } else if (value.length > 64) {
                            exception.addFieldErrorAndResponseCodes(segment + key, AppErrorCodes.CUSTOM_FIELD_VALUE_INVALID_LENGTH)
                        }
                    }
                }
            }
        };

        return exports;
    }
));
